@@ -17,8 +17,9 @@ fn basic_copy() {
     let src_file_path = src.path().join(file_name);
     fs::write(&src_file_path, file_content).unwrap();
 
-    let options = CopyOptions::default();
-    let stats = copy_tree(src.path(), dest.path(), &options).unwrap();
+    let stats = CopyOptions::default()
+        .copy_tree(src.path(), dest.path())
+        .unwrap();
 
     let dest_file_path = &dest.path().join(file_name);
     assert_eq!(fs::read(&dest_file_path).unwrap(), file_content);
@@ -42,8 +43,10 @@ fn larger_file_with_small_buffer_causes_multiple_reads() {
 
     fs::write(&src.path().join(file_name), &file_content).unwrap();
 
-    let options = CopyOptions::new().with_copy_buffer_size(small_copy_buffer_size);
-    let stats = copy_tree(src.path(), dest.path(), &options).unwrap();
+    let stats = CopyOptions::new()
+        .copy_buffer_size(small_copy_buffer_size)
+        .copy_tree(src.path(), dest.path())
+        .unwrap();
 
     assert_eq!(
         fs::read(&dest.path().join(file_name)).unwrap(),
@@ -70,7 +73,9 @@ fn subdirs() {
     let file_content = b"some file content\n";
     fs::write(&src.path().join("a/aa/aaafile"), &file_content).unwrap();
 
-    let stats = copy_tree(src.path(), dest.path(), &CopyOptions::default()).unwrap();
+    let stats = CopyOptions::default()
+        .copy_tree(src.path(), dest.path())
+        .unwrap();
 
     assert_eq!(
         fs::read(&dest.path().join("a/aa/aaafile")).unwrap(),
@@ -88,7 +93,9 @@ fn subdirs() {
 #[test]
 fn clean_error_on_nonexistent_source() {
     let dest = tempfile::tempdir().unwrap();
-    let err = copy_tree(Path::new("nothing"), &dest.path(), &CopyOptions::new()).unwrap_err();
+    let err = CopyOptions::new()
+        .copy_tree(Path::new("nothing"), &dest.path())
+        .unwrap_err();
     println!("err = {:#?}", err);
     assert!(err.path().starts_with("nothing"));
     assert_eq!(err.kind(), ErrorKind::ReadDir);
@@ -100,7 +107,9 @@ fn create_destination_by_default() {
     let empty_src = tempfile::tempdir().unwrap();
     let dest_parent = tempfile::tempdir().unwrap();
     let dest = dest_parent.path().join("nonexistent_child");
-    let stats = copy_tree(&empty_src.path(), &dest, &CopyOptions::new()).unwrap();
+    let stats = CopyOptions::new()
+        .copy_tree(&empty_src.path(), &dest)
+        .unwrap();
     assert!(dest.is_dir());
     assert_eq!(stats.dirs, 1);
     assert_eq!(stats.files, 0);
@@ -111,12 +120,10 @@ fn create_destination_when_requested() {
     let empty_src = tempfile::tempdir().unwrap();
     let dest_parent = tempfile::tempdir().unwrap();
     let dest = dest_parent.path().join("nonexistent_child");
-    let stats = copy_tree(
-        &empty_src.path(),
-        &dest,
-        &CopyOptions::new().create_destination(true),
-    )
-    .unwrap();
+    let stats = CopyOptions::new()
+        .create_destination(true)
+        .copy_tree(&empty_src.path(), &dest)
+        .unwrap();
     assert!(dest.is_dir());
     assert_eq!(stats.dirs, 1);
     assert_eq!(stats.files, 0);
@@ -128,12 +135,10 @@ fn optionally_destination_must_exist() {
     // But, for now, it must.
     let dest_parent = tempfile::tempdir().unwrap();
     let dest = dest_parent.path().join("nonexistent_child");
-    let err = copy_tree(
-        Path::new("src"),
-        &dest,
-        &CopyOptions::new().create_destination(false),
-    )
-    .unwrap_err();
+    let err = CopyOptions::new()
+        .create_destination(false)
+        .copy_tree(Path::new("src"), &dest)
+        .unwrap_err();
     println!("err = {:#?}", err);
     assert!(err.path().starts_with(&dest));
     assert_eq!(err.kind(), ErrorKind::WriteFile);
@@ -144,7 +149,9 @@ fn optionally_destination_must_exist() {
 #[test]
 fn clean_error_failing_to_copy_devices() {
     let dest = tempfile::tempdir().unwrap();
-    let err = copy_tree(&Path::new("/dev"), dest.path(), &Default::default()).unwrap_err();
+    let err = CopyOptions::new()
+        .copy_tree(&Path::new("/dev"), dest.path())
+        .unwrap_err();
     println!("{:#?}", err);
     assert_eq!(err.kind(), ErrorKind::UnsupportedFileType);
     assert_eq!(err.io_error().kind(), io::ErrorKind::Unsupported);
@@ -158,7 +165,9 @@ fn copy_dangling_symlink() {
     let src = tempfile::tempdir().unwrap();
     let dest = tempfile::tempdir().unwrap();
     std::os::unix::fs::symlink("dangling target", src.path().join("a_link")).unwrap();
-    let stats = copy_tree(src.path(), dest.path(), &CopyOptions::new()).unwrap();
+    let stats = CopyOptions::new()
+        .copy_tree(src.path(), dest.path())
+        .unwrap();
     println!("{:#?}", stats);
     assert_eq!(
         stats,
@@ -168,6 +177,96 @@ fn copy_dangling_symlink() {
             symlinks: 1,
             file_bytes: 0,
             file_buffer_reads: 0,
+            filtered_out: 0,
         }
     );
 }
+
+#[test]
+fn filter_by_path() {
+    let src = tempfile::tempdir().unwrap();
+    let dest = tempfile::tempdir().unwrap();
+
+    fs::create_dir(&src.path().join("a")).unwrap();
+    fs::create_dir(&src.path().join("b")).unwrap();
+    fs::create_dir(&src.path().join("b/bb")).unwrap();
+    fs::create_dir(&src.path().join("a").join("aa")).unwrap();
+
+    let file_content = b"some file content\n";
+    fs::write(&src.path().join("a/aa/aaafile"), &file_content).unwrap();
+
+    fn not_b(path: &Path, _: &fs::DirEntry) -> cp_r::Result<bool> {
+        Ok(path != Path::new("b"))
+    }
+    let stats = CopyOptions::new()
+        .filter(not_b)
+        .copy_tree(src.path(), dest.path())
+        .unwrap();
+
+    assert_eq!(
+        fs::read(&dest.path().join("a/aa/aaafile")).unwrap(),
+        file_content
+    );
+    assert!(!dest.path().join("b").exists());
+    assert_eq!(
+        stats,
+        CopyStats {
+            files: 1,
+            file_bytes: file_content.len() as u64,
+            dirs: 2,
+            symlinks: 0,
+            file_buffer_reads: 1,
+            filtered_out: 1,
+        }
+    );
+}
+
+#[test]
+fn filter_by_mut_closure() {
+    let src = tempfile::tempdir().unwrap();
+    let dest = tempfile::tempdir().unwrap();
+
+    fs::create_dir(&src.path().join("a")).unwrap();
+    fs::create_dir(&src.path().join("b")).unwrap();
+    fs::create_dir(&src.path().join("b/bb")).unwrap();
+    fs::create_dir(&src.path().join("a").join("aa")).unwrap();
+
+    let file_content = b"some file content\n";
+    fs::write(&src.path().join("a/aa/aaafile"), &file_content).unwrap();
+
+    // Filter paths and also collect all the paths we've seen, as an example of a filter
+    // that's more than a simple function pointer.
+    let mut filter_seen_paths: Vec<String> = Vec::new();
+    let stats = CopyOptions::default()
+        .filter(|path: &Path, _de| {
+            filter_seen_paths.push(path.to_str().unwrap().replace('\\', "/"));
+            Ok(path != Path::new("b"))
+        })
+        .copy_tree(src.path(), dest.path())
+        .unwrap();
+
+    assert_eq!(
+        fs::read(&dest.path().join("a/aa/aaafile")).unwrap(),
+        file_content
+    );
+    assert!(!dest.path().join("b").exists());
+    assert_eq!(
+        stats,
+        CopyStats {
+            files: 1,
+            file_bytes: file_content.len() as u64,
+            dirs: 2,
+            symlinks: 0,
+            file_buffer_reads: 1,
+            filtered_out: 1,
+        }
+    );
+    // The order in which entries are seen is not guaranteed, and in practice
+    // will be partly determined by the unpredictable order that the filesystem
+    // returns directory entries.
+    //
+    // "b" is seen (because the filter records it before filtering it out), but
+    // b's children are not visited.
+    filter_seen_paths.sort_unstable();
+    assert_eq!(filter_seen_paths, ["a", "a/aa", "a/aa/aaafile", "b"]);
+}
